"""
io - Module for I/O operations

AUTHOR:

- JENS PUTZKA (2010-04-21): initial version

DATE:
- 2010-10-06

VERSION:
- 1.5 - 2010-10-06
- 1.0 - 2010-04-21


"""


#*****************************************************************************
#       Copyright (C) 2009 - JENS PUTZKA <putzka@mpim-bonn.mpg.de>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************

#===============================================================================
# Imports
#===============================================================================

from os import access, F_OK, W_OK, sep
from sage.structure.sage_object import save,load
from config import BASEPATH



#===============================================================================
# check_opath ()
#===============================================================================

def checkopath (opath):
    """
    INPUT:
        opath - absolute path as string
        
        
    OUTPUT:
        Returns 
        True is path exists and is writeable
        False if it does not exist
        Raise IOError if exists but is not writeable
    """
    
    if not access(opath, F_OK):
        return False
    elif access(opath, W_OK):
        return True
    else:
        raise IOError, "Path %s exists but ins not writeable." % tmp_path

#===============================================================================
# check_ipath ()
#===============================================================================

def checkipath (ipath):
    """
    INPUT:
        ipath - absolute path as string
        
    OUTPUT:
        Returns 
        True is path exists and is readable
        False if it does not exist
        Raise IOError if exists but is not readable
    """
    
    if not access(ipath, F_OK):
        return False
    elif access(ipath, R_OK):
        return True
    else:
        raise IOError, "Path %s exists but ins not readable." % tmp_path


#===============================================================================
# save_obj2file ()
#===============================================================================
def saveobj2file (obj,filename,basedir = None, dir= 'output' , extension='sobj'):
    """
    saveobj2file - save object filename.extension
    
    Saves a give Sage object, or to be precise the dump string of the object to disk.
    By default it uses the standard Sage object type as file type.
    
    Note:
    It might be useful to include checks whether the paths exist. 
    Since otherwise    
    
    INPUT::
    object    = the thing you want to save
    filename  =  string
    basedir   = BASEPATH (optional)
    dir       = 'output' (optional)
    extension = 'sobj'   (optional)
    
    
    """

    if basedir == None:
        _outfile = BASEPATH +sep+dir + sep+filename +'.'+extension
    else:
        _outfile = basedir+sep+dir +sep+ filename +'.'+extension      
    print "Save to %s" %_outfile
    if not checkopath( basedir+sep+dir +sep):
        raise IOError, "Path does not exist."
    save (obj, _outfile)


#===============================================================================
# savelist2file ()
#===============================================================================

def savelist2file(lst,filename, basedir = None, dir= 'output' ,extension='txt'):
    """
    Saves a list of object to file, see documentation of saveobj2file
    """
    if basedir == None:
        _outfile = open(BASEPATH +sep+dir + sep+filename +'.'+extension, 'w')
    else:
        _outfile = open(basedir+sep+dir + sep+filename +'.'+extension, 'w')
    
    for entry in lst:
        _outfile.write(str(entry));
        _outfile.write('\n\n');
    _outfile.close();




#===============================================================================
#  loadobj  ()
#===============================================================================

def loadobj (filename, basedir = None, dir= 'output' , extension='sobj'):
    """
    loadobj - load object filename.extension
    
    By default it loads the standard format in which the Sage objects
    are stored to disk.
    
    INPUT::
    
    filename  =  string
    basedir   = BASEPATH (optional)
    dir       = 'output' (optional)
    extension = 'sobj' (optional)
    
    """
    if basedir == None:
        _infile = BASEPATH +sep+dir + sep+filename +'.'+extension
    else:
        _infile = basedir+sep+dir +sep+ filename +'.'+extension  

    return load(_infile)
    