"""
MatrixOperations - Module to decribe some additional matrix operations

AUTHOR:

- JENS PUTZKA (2011-02-03): initial version

DATE:
- 2011-02-03

VERSION:
- 1.0 - 2011-02-03


"""
#*****************************************************************************
#       Copyright (C) 2011 - JENS PUTZKA <putzka@mpim-bonn.mpg.de>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************

print "\n=========================================================";
print '|  Importing MatrixOperations Package  (c) 2011         |';
print '|                                                       |'
print '|    by Jens Putzka MPI for Mathematics Bonn            |'
print "=========================================================";

from sage.functions.other import factorial
from scipy.linalg import expm

#===============================================================================
# is_diagonal_matrix
#===============================================================================    
    
def is_diagonal_matrix(mat):
    """
    Returns True if mat is a diagonal matrix
    """
    n_cols = mat.ncols()
    n_rows = mat.ncols()
    return all([mat[i,j]== 0 for i in range(n_cols) for j in range(n_rows) if i!=j])

#===============================================================================
# expmat
#===============================================================================

def expmat(mat, method=-1):
    """
    Computes exponential of a nilpotent matrix
    methods:
    -1 = naive
    0 = uses scipy.linalg.expm without type conversion Ring id RDF
    1 = 0 but with type conversion to typ of nbase_ring of input matrix
    
    """
    
    if method== -1:
        _m   = mat;
        _out = _m.parent().one_element()
        _n   = 1
        while not _m.is_zero():
            _out= _out+ _m/factorial(_n)
        
            _n += 1
            _m = _m*mat
    elif method == 0: 
     
        _exp_mat = expm(mat)
    
        _out = matrix(mat.base_ring(),_exp_mat)
        print "Test:%s" %(_out.base_ring()==mat.base_ring())
    elif method == 1:
        _exp_mat = expm(mat).tolist()
    
        _out = matrix(mat.base_ring(),_exp_mat)
        print "Test:%s" %(_out.base_ring()==mat.base_ring())   
    return _out  